<?php

namespace Drp\SimpleCms\Http\Controllers\Admin;

use Drp\SimpleCms\Contracts\ContentFactory;
use Drp\SimpleCms\Contracts\ContentManager;
use Drp\SimpleCms\Contracts\ContentRequest;
use Drp\SimpleCms\Exceptions\NoUpdatesMade;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Http\RedirectResponse;
use Illuminate\Routing\Controller;
use Illuminate\Validation\ValidationException;
use Illuminate\View\View;

class ManageContentController extends Controller
{
    use AuthorizesRequests;

    /**
     * @var ContentFactory
     */
    private $contentFactory;

    /**
     * @var ContentManager
     */
    private $contentManager;

    public function __construct(
        ContentFactory $contentFactory,
        ContentManager $contentManager
    ) {
        $this->contentFactory = $contentFactory;
        $this->contentManager = $contentManager;
    }

    public function index(string $contentType, ?int $contentId = null): View
    {
        $content = $this->contentFactory->latestVersionByTypeAndContentId($contentType, $contentId);

        return view('simpleCms::manage', compact('content', 'contentType'));
    }

    public function store(ContentRequest $request, string $contentType): RedirectResponse
    {
        try {
            $request->validate($this->contentFactory->makeContentClass($contentType));

            $content = $this->contentManager->createContent($contentType, request()->input());

            flash()->success(trans('simpleCms::admin.created', [
                'label' => ucfirst(content_label($contentType, false)),
            ]));

            return redirect($content->adminEditUrl());
        } catch (ValidationException $exception) {
            flash()->error(trans('validation.error'));
            throw $exception;
        }
    }

    public function update(ContentRequest $request, string $contentType, int $contentId): RedirectResponse
    {
        $content = $this->contentFactory->latestVersionByTypeAndContentId($contentType, $contentId);

        try {
            $request
                ->prepareInput($content)
                ->validate($content);

            $this->contentManager->updateContent($content, $request->all());
            flash()->success(trans('simpleCms::admin.updated', [
                'label' => ucfirst(content_label($contentType, false)),
            ]));
        } catch (NoUpdatesMade $exception) {
            flash()->info(trans('simpleCms::content.no-change'));
        } catch (ValidationException $exception) {
            flash()->error(trans('validation.error'));
            throw $exception;
        }

        return redirect($content->adminEditUrl());
    }

    public function destroy(string $contentType, int $contentId): RedirectResponse
    {
        $content = $this->contentFactory->latestVersionByTypeAndContentId($contentType, $contentId);

        $this->authorize('delete', $content);
        $content->destroyAllVersion();

        flash()->success(trans('simpleCms::admin.deleted', ['label' => 'Content']));

        return redirect()->route('admin.content.index', [
            'type' => $contentType,
        ]);
    }
}
